/**
 * \brief These are various functions for ensuring uniform log messaging
 *
 * \authors Brian Jensen
 *          <br>
 *          {jensen}\@in.tum.de
 
 * \ingroup PetSpectFusion
 * \version 1.30
 * \date 26.10.2009
 *
 * \par License:
 * Copyright (c) 2007 - 2009,
 * This programm was created as part of a student research project in cooperation
 * with the Department for Computer Science, Chair XVI
 * and the Nuklearmedizinische Klinik, Klinikum Rechts der Isar
 *
 * <br>
 * <br>
 * All rights reserved.
 * <br>
 * <br>
 * See <a href="COPYRIGHT.txt">COPYRIGHT.txt</a> for details.
 * <br>
 * <br>
 * This software is distributed WITHOUT ANY WARRANTY; without even 
 * <br>
 * the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR 
 * <br>
 * PURPOSE.  See the <a href="COPYRIGHT.txt">COPYRIGHT.txt</a> notice
 * for more information.
 *
 */

#ifndef LOG_H
#define LOG_H

#include <string>
#include <sstream>

namespace logging {

	/**
	 *	Functions create a formated output string to stdout with a time stamp
	 */
	void logMessage(std::string& msg);
	void logMessage(const char* msg);
	void logMessage(std::ostringstream& msg);

	/**
	 *	Functions create a formatted output string to stderr with a time stamp
	 */
	void logError(std::string& msg);
	void logError(const char* msg);
	void logError(std::ostringstream& msg);

	/**
	 *	Functions are responsible redirecting output to a log file
	 */
	void enableFileLogging(std::string& fileName);
	void disableFileLogging();
	bool fileLoggingEnabled();
	std::string& getLogFileName();	
	
}
	
#endif