#ifndef TYPEDEFS_H
#define TYPEDEFS_H

/**
 *	$Id: typedefs.h 39:803510a3c9e1 2009-11-18 20:37 +0100 jensen $
 *  $Lastlog: large refactoring for consistent initialization, unit test added, refs #2 #17 #18 $
 *
 *\brief This file sets various default types and values used throughout the application
 *
 * \authors Brian Jensen
 *          <br>
 *          {jensen}\@cs.tum.edu
 *
 * \ingroup PetSpectFusion
 * \version 1.0
 * \date 16.12.2007
 *
 * \par License:
 * Copyright (c) 2007 - 2009,
 * This programm was created as part of a student research project in cooperation
 * with the Department for Computer Science, Chair XVI
 * and the Nuklearmedizinische Klinik, Klinikum Rechts der Isar
 *
 */


//Itk IO includes
#include "itkGDCMImageIO.h"
#include "itkImage.h"
#include "itkImageSeriesWriter.h"
#include "itkImageFileReader.h"
#include "itkImageFileWriter.h"
#include "itkMetaImageIO.h"
#include "itkMetaDataObject.h"
#include "itkNumericSeriesFileNames.h"

//itk filtering includes
#include "itkCastImageFilter.h"
#include "itkCheckerBoardImageFilter.h"
#include "itkExtractImageFilter.h"
//#include "itkIntensityWindowingImageFilter.h"
#include "itkMinimumMaximumImageCalculator.h"
#include "itkNormalizeImageFilter.h"
#include "itkResampleImageFilter.h"
#include "itkRescaleIntensityImageFilter.h"
#include "itkSubtractImageFilter.h"
#include "itkShiftScaleImageFilter.h"

//itk registration includes
#include "itkImageRegistrationMethod.h"
#include "itkLinearInterpolateImageFunction.h"
#include "itkMultiResolutionImageRegistrationMethod.h"
#include "itkRecursiveMultiResolutionPyramidImageFilter.h"
#include "itkMutualInformationImageToImageMetric.h"
#include "itkMattesMutualInformationImageToImageMetric.h"
#include "itkGradientDescentOptimizer.h"
#include "itkVersorRigid3DTransform.h"
#include "itkVersorRigid3DTransformOptimizer.h"
#include "itkCenteredTransformInitializer.h"
#include "itkCenteredVersorTransformInitializer.h"

//itk utilities include
#include "itkRealTimeClock.h"

//setup internal image types
typedef float InternalPixelType;
const unsigned int Dimension = 3;
typedef itk::Image<InternalPixelType, Dimension> InternalImageType;

//setup external image types
typedef float OutputPixelType; //initially int, changed to float by Martina/Axel due to 0 output error
typedef itk::Image<OutputPixelType, Dimension> OutputImageType;

//now set up the types for the registration
typedef itk::VersorRigid3DTransform< double >  TransformType;
typedef itk::VersorRigid3DTransformOptimizer  OptimizerType;
typedef itk::ImageRegistrationMethod<InternalImageType, InternalImageType> RegistrationType;
typedef OptimizerType::ScalesType OptimizerScalesType;
typedef TransformType::VersorType VersorType;

//Types for the multi resolution registration
typedef itk::MultiResolutionImageRegistrationMethod< InternalImageType, InternalImageType> MultiResRegistrationType;
typedef itk::MultiResolutionPyramidImageFilter<InternalImageType, InternalImageType> InternalImagePyramidType;

//Types for converting the rotation parameters
typedef itk::Array<double> ParametersType;
typedef itk::Array<float> RotationType;

//resampler used for displaying the images
typedef unsigned char DisplayPixelType;
typedef itk::Image<DisplayPixelType, Dimension> DisplayImageType;
typedef itk::ResampleImageFilter<InternalImageType, InternalImageType> DisplayResamplerType;
typedef itk::RescaleIntensityImageFilter<InternalImageType, DisplayImageType> DisplayRescalerType;
typedef itk::ShiftScaleImageFilter<InternalImageType, DisplayImageType> DisplayScalerType;
typedef itk::CheckerBoardImageFilter<DisplayImageType> CheckerBoardType;

//protypes used by main
void print_help(const char* name);

//Multhreading parameter, initialized to 0 which defaults to the number of processors on the system
extern int thread_count;

//registration result type
typedef struct
{
	double metricVal;
	int iteration;
	ParametersType params;
	int error;
} RegistrationResultType;

#define DEFAULT_BINS 55
#define DEFAULT_MINSTEP 0.3
#define DEFAULT_MAXSTEP 3.3
#define DEFAULT_MAXITER 50
#define DEFAULT_SAMPLERATE 0.05
#define DEFAULT_XTRANS_SCALE 0.00003
#define DEFAULT_YTRANS_SCALE 0.00003
#define DEFAULT_ZTRANS_SCALE 0.00003
#define DEFAULT_XROT_SCALE 1.0
#define DEFAULT_YROT_SCALE 1.0
#define DEFAULT_ZROT_SCALE 1.0
#define DEFAULT_DO_MULTIRES false
#define DEFAULT_MULTIRES_LEVELS 3
#define DEFAULT_ZOOM_FACTOR 2

#define DEFAULT_WINDOW_MAX 1.0
#define DEFAULT_WINDOW_MIN 0.0
#define DEFAULT_CHECKER_SQUARES 5

#endif

