//
//  $Id: RegObserver.cxx 21:03f3a7e898aa 2009-11-04 11:08 +0100 jensen $
//  $Lastlog: Added more keyword tags $
//
//  RegObserver.cxx
//  PetSpectFusion
//
//	Created 16/12/2007, Implements the observer pattern for keeping track of the registration
//  Copyright (c) 2007 - 2009 Brian Jensen. All rights reserved.
//

#include "Log.h"
#include "RegObserver.h"

#include <sstream>

using namespace logging;

void CommandIterationUpdate::Execute(itk::Object *caller, const itk::EventObject & event)
{
	OptimizerPointer optimizer = 
	dynamic_cast< OptimizerPointer >( caller );
		
	if(stopReg)
	{
		logMessage("stopping the optimizer...");
		optimizer->StopOptimization();
	}
	
	Execute( (const itk::Object *)caller, event);
}

void CommandIterationUpdate::Execute(const itk::Object * object, const itk::EventObject & event)
{	
	ConstOptimizerPointer optimizer = 
		dynamic_cast< ConstOptimizerPointer >( object );
	if( ! itk::IterationEvent().CheckEvent( &event ) )
	{
		return;
	}
	
	std::ostringstream ostring(std::ostringstream::out);

	ostring << optimizer->GetCurrentIteration() << "   ";
	ostring << optimizer->GetValue() << "   ";
	ostring << optimizer->GetCurrentPosition();
	logMessage(ostring);
					
	if(plotData)
	{
		itk::Array<double> pos = optimizer->GetCurrentPosition();
		anglesFile << optimizer->GetCurrentIteration() << " " << pos.GetElement(0) << " " << pos.GetElement(1) << " " << pos.GetElement(2) << std::endl;
		transFile << optimizer->GetCurrentIteration() << " " << pos.GetElement(3) <<  " " << pos.GetElement(4) << " " << pos.GetElement(5) << std::endl;
		metricFile << optimizer->GetCurrentIteration() << " " << optimizer->GetValue() << std::endl;
	}
	
	if(regThread)
	{
		ParametersType params = optimizer->GetCurrentPosition();
		regThread->registrationUpdate(params, optimizer->GetValue(), optimizer->GetCurrentIteration());
	}

}


void CommandIterationUpdate::enablePlotting(const char* fileName)
{
	plotData = true;
	
	std::string baseName = fileName;
	std::string transFileName = baseName + "_trans.dat";
	transFile.open(transFileName.c_str());
		
	std::string anglesFileName = baseName + "_angles.dat";
	anglesFile.open(anglesFileName.c_str());
		
	std::string metricFileName = baseName + "_metric.dat";
	metricFile.open(metricFileName.c_str());
		
	//write a meta file so that the gnuplot script knows which files to load
	std::string plotNames = baseName + "_files.plt";
	std::ofstream plotNamesFile;
	plotNamesFile.open(plotNames.c_str());
	plotNamesFile << "anglesFile = \"" << anglesFileName << "\"" << std::endl;
	plotNamesFile << "transFile = \"" << transFileName << "\"" << std::endl;
	plotNamesFile << "metricFile = \"" << metricFileName << "\"" <<std::endl;
	plotNamesFile.close();
	
}

void CommandIterationUpdate::closePlotFiles()
{
	transFile.close();
	anglesFile.close();
	metricFile.close();
}

void CommandIterationUpdate::setDisplayThread(QRegistrationThread* thread)
{
	regThread = thread;
}

void CommandIterationUpdate::stopRegistration()
{
	stopReg = true;
}
