//
//  $Id: RegSettingsDialog.cxx 27:1beb6e7bc8c7 2009-11-11 18:18 +0100 jensen $
//  $Lastlog: added auto calibration and auto updating for refs #9 $
//
//  RegSettingsDialog.cxx
//  PetSpectFusion
//
//	Created 16/12/2007, Implements a dialog for changing various settings in PetSpectFusion
//  Copyright (c) 2007 - 2009 Brian Jensen. All rights reserved.
//

#include "RegSettingsDialog.h"

RegSettingsDialog::RegSettingsDialog(QWidget *parent) : QDialog(parent)
{
	QGridLayout *settingsGrid = new QGridLayout;
	QLabel* binsLabel = new QLabel(tr("Bins"));
	settingsGrid->addWidget(binsLabel, 0, 0);
	binsBox = new QLineEdit;
	binsBox->setText(QString::number(DEFAULT_BINS));
	binsBox->setInputMask(QString("0000"));
	settingsGrid->addWidget(binsBox, 0 , 1);
	QLabel *samplesLabel = new QLabel(tr("Sample Rate"));
	settingsGrid->addWidget(samplesLabel, 0, 2);
	samplesBox = new QLineEdit;
	samplesBox->setInputMask(QString("0.000"));
	samplesBox->setText(QString::number(DEFAULT_SAMPLERATE));
	settingsGrid->addWidget(samplesBox, 0, 3);
	
	QLabel *minStepLabel = new QLabel(tr("Min Step Size"));
	settingsGrid->addWidget(minStepLabel, 1, 0);
	minSizeBox = new QLineEdit;
	minSizeBox->setInputMask(QString("00.000"));
	minSizeBox->setText(QString::number(DEFAULT_MINSTEP));
	settingsGrid->addWidget(minSizeBox, 1, 1);
	QLabel* maxStepLabel = new QLabel(tr("Max Step Size"));
	settingsGrid->addWidget(maxStepLabel, 1, 2);
	maxSizeBox = new QLineEdit;
	maxSizeBox->setInputMask(QString("00.000"));
	maxSizeBox->setText(QString::number( DEFAULT_MAXSTEP));
	settingsGrid->addWidget(maxSizeBox, 1, 3);
	
	QGroupBox *settingsBox = new QGroupBox(QString("Parameters"));
	settingsBox->setLayout(settingsGrid);
	
	QGridLayout *loggingGrid = new QGridLayout;
	QLabel* loggingLabel = new QLabel(tr("Enable Logging"));
	loggingGrid->addWidget(loggingLabel, 0, 0);
	loggingBox = new QCheckBox;
	loggingGrid->addWidget(loggingBox, 0, 1);
	logNameBox = new QLineEdit;
	loggingGrid->addWidget(logNameBox, 0, 2);
	loggingButton = new QPushButton(tr("Log file"));
	loggingButton->setSizePolicy(QSizePolicy::Fixed, QSizePolicy::Fixed);
	loggingGrid->addWidget(loggingButton, 0, 3);
	
	QLabel* plotsLabel = new QLabel(tr("Enable Plots"));
	loggingGrid->addWidget(plotsLabel, 1, 0);
	plotsBox = new QCheckBox;
	loggingGrid->addWidget(plotsBox, 1, 1);
	plotsNameBox = new QLineEdit;
	loggingGrid->addWidget(plotsNameBox, 1, 2);
	plotsButton = new QPushButton(tr("Plots file"));
	plotsButton->setSizePolicy(QSizePolicy::Fixed, QSizePolicy::Fixed);
	loggingGrid->addWidget(plotsButton, 1, 3);
	
	QGroupBox *logplotBox = new QGroupBox(QString("Logging / Plotting"));
	logplotBox->setLayout(loggingGrid);
	
	//optimizer scales grid
	QGridLayout *scalesGrid = new QGridLayout;
	QLabel *xLabel = new QLabel(tr("X"));
	scalesGrid->addWidget(xLabel, 0, 1);
	QLabel *yLabel = new QLabel(tr("Y"));
	scalesGrid->addWidget(yLabel, 0, 2);
	QLabel *zLabel = new QLabel(tr("Z"));
	scalesGrid->addWidget(zLabel, 0, 3);
	
	QLabel *rotLabel = new QLabel(tr("Rotation"));
	scalesGrid->addWidget(rotLabel, 1, 0);
	xRotBox = new QLineEdit;
	xRotBox->setInputMask(QString("00.0000000"));
	scalesGrid->addWidget(xRotBox, 1, 1);
	yRotBox = new QLineEdit;
	yRotBox->setInputMask(QString("00.0000000"));
	scalesGrid->addWidget(yRotBox, 1, 2);
	zRotBox = new QLineEdit;
	zRotBox->setInputMask(QString("00.0000000"));
	scalesGrid->addWidget(zRotBox, 1, 3);
	
	QLabel *translLabel = new QLabel(tr("Translation"));
	scalesGrid->addWidget(translLabel, 2, 0);
	xTransBox = new QLineEdit;
	xTransBox->setInputMask(QString("00.0000000"));
	scalesGrid->addWidget(xTransBox, 2, 1);
	yTransBox = new QLineEdit;
	yTransBox->setInputMask(QString("00.0000000"));
	scalesGrid->addWidget(yTransBox, 2, 2);
	zTransBox = new QLineEdit;
	zTransBox->setInputMask(QString("00.0000000"));
	scalesGrid->addWidget(zTransBox, 2, 3);
	
	QGroupBox *scalesBox = new QGroupBox(QString("Optimizer Scales"));
	scalesBox->setLayout(scalesGrid);
	
	//buttons
	QGridLayout *buttonsGrid = new QGridLayout;
	QPushButton *okButton = new QPushButton(tr("Ok"));
	okButton->setSizePolicy(QSizePolicy::Fixed, QSizePolicy::Fixed);
	buttonsGrid->addWidget(okButton, 0, 0);
	connect(okButton, SIGNAL(clicked()), this, SLOT(accept()));
	QPushButton *cancelButton = new QPushButton(tr("Cancel"));
	cancelButton->setSizePolicy(QSizePolicy::Fixed, QSizePolicy::Fixed);
	cancelButton->setDefault(true);
	buttonsGrid->addWidget(cancelButton, 0, 1);
	connect(cancelButton, SIGNAL(clicked()), this, SLOT(reject()));
	
	connect(loggingButton, SIGNAL(clicked()), this, SLOT(chooseLogFile()));
	connect(plotsButton, SIGNAL(clicked()), this, SLOT(choosePlotsFile()));
	
	QGridLayout *threadsGrid = new QGridLayout;
	QLabel *threadsLabel = new QLabel(tr("Thread Count"));
	threadsGrid->addWidget(threadsLabel, 0, 0);
	threadsBox = new QLineEdit;
	threadsGrid->addWidget(threadsBox, 0, 1);
	QGroupBox *threadingBox = new QGroupBox(QString("Multithreading"));
	threadingBox->setLayout(threadsGrid);
	
	QVBoxLayout *box = new QVBoxLayout;
	box->addWidget(scalesBox);
	box->addWidget(settingsBox);
	box->addWidget(logplotBox);
	box->addWidget(threadingBox);
	box->addLayout(buttonsGrid);
	
	setLayout(box);
	
	setWindowTitle(tr("Options"));

}

void RegSettingsDialog::chooseLogFile()
{
	QString fileName = QFileDialog::getSaveFileName(this, tr("Log File"),
                            "", tr("Text files (*.txt)"));
	logNameBox->setText(fileName);
}

void RegSettingsDialog::choosePlotsFile()
{
	QString fileName = QFileDialog::getSaveFileName(this, tr("Plot file base"),
                            "", tr("Just a prefix, endings added automatically"));
	plotsNameBox->setText(fileName);
}

